@@ -1,10 +1,9 @@
 import logging
 
 import numpy as np
-from scipy.spatial.transform import Rotation
 
 from aspire.abinitio import CLOrient3D
-from aspire.utils import ensure
+from aspire.utils import Rotation
 from aspire.utils.matlab_compat import stable_eigsh
 
 logger = logging.getLogger(__name__)
@@ -24,15 +23,25 @@ class CLSyncVoting(CLOrient3D):
     Journal of Structural Biology, 169, 312-322 (2010).
     """
 
-    def __init__(self, src, n_rad=None, n_theta=None):
+    def __init__(self, src, n_rad=None, n_theta=360, max_shift=0.15, shift_step=1):
         """
         Initialize an object for estimating 3D orientations using synchronization matrix
 
         :param src: The source object of 2D denoised or class-averaged images with metadata
         :param n_rad: The number of points in the radial direction
-        :param n_theta: The number of points in the theta direction
+        :param n_theta: The number of points in the theta direction.
+            Default is 360.
+        :param max_shift: Determines maximum range for shifts as a proportion
+            of the resolution. Default is 0.15.
+        :param shift_step: Resolution for shift estimation in pixels. Default is 1 pixel.
         """
-        super().__init__(src, n_rad=n_rad, n_theta=n_theta)
+        super().__init__(
+            src,
+            n_rad=n_rad,
+            n_theta=n_theta,
+            max_shift=max_shift,
+            shift_step=shift_step,
+        )
         self.syncmatrix = None
 
     def estimate_rotations(self):
@@ -44,8 +53,8 @@ class CLSyncVoting(CLOrient3D):
 
         S = self.syncmatrix
         sz = S.shape
-        ensure(sz[0] == sz[1], "syncmatrix must be a square matrix.")
-        ensure(sz[0] % 2 == 0, "syncmatrix must be a square matrix of size 2Kx2K.")
+        assert sz[0] == sz[1], "syncmatrix must be a square matrix."
+        assert sz[0] % 2 == 0, "syncmatrix must be a square matrix of size 2Kx2K."
 
         n_img = sz[0] // 2
 
@@ -144,7 +153,7 @@ class CLSyncVoting(CLOrient3D):
         sz = clmatrix.shape
         n_theta = self.n_theta
 
-        ensure(sz[0] == sz[1], "clmatrix must be a square matrix.")
+        assert sz[0] == sz[1], "clmatrix must be a square matrix."
 
         n_img = sz[0]
         S = np.eye(2 * n_img, dtype=self.dtype).reshape(n_img, 2, n_img, 2)
@@ -179,7 +188,7 @@ class CLSyncVoting(CLOrient3D):
 
         rots = self._rotratio_eulerangle_vec(clmatrix, i, j, good_k, n_theta)
 
-        if rots.shape[0] > 0:
+        if rots is not None:
             rot_mean = np.mean(rots, 0)
             # The error to mean value can be calculated as
             #    rot_block = rots[:2, :2]
@@ -230,14 +239,16 @@ class CLSyncVoting(CLOrient3D):
 
         # Calculate the cos values of rotation angles between i an j images for good k images
         c_alpha, good_idx = self._get_cos_phis(cl_diff1, cl_diff2, cl_diff3, n_theta)
+        if len(c_alpha) == 0:
+            return None
         alpha = np.arccos(c_alpha)
 
-        # Convert the Euler angles with ZXZ conversion to rotation matrices
+        # Convert the Euler angles with ZYZ conversion to rotation matrices
         angles = np.zeros((alpha.shape[0], 3))
-        angles[:, 0] = clmatrix[i, j] * 2 * np.pi / n_theta - np.pi
+        angles[:, 0] = clmatrix[i, j] * 2 * np.pi / n_theta + np.pi / 2
         angles[:, 1] = alpha
-        angles[:, 2] = np.pi - clmatrix[j, i] * 2 * np.pi / n_theta
-        r = Rotation.from_euler("ZXZ", angles).as_matrix()
+        angles[:, 2] = -np.pi / 2 - clmatrix[j, i] * 2 * np.pi / n_theta
+        r = Rotation.from_euler(angles).matrices
 
         return r[good_idx, :, :]
 
@@ -324,7 +335,7 @@ class CLSyncVoting(CLOrient3D):
         angles_hist = np.sum(
             np.exp(
                 (2 * np.multiply.outer(angles, angles_grid) - squared_values)
-                / (2 * sigma ** 2)
+                / (2 * sigma**2)
             ),
             0,
         )
