@@ -3,9 +3,9 @@ import logging
 import numpy as np
 from scipy.sparse.linalg import LinearOperator, cg
 
-from aspire.basis.basis_utils import num_besselj_zeros
-from aspire.utils import ensure, mdim_mat_fun_conj
-from aspire.utils.matlab_compat import m_reshape
+from aspire.image import Image
+from aspire.utils import mdim_mat_fun_conj
+from aspire.volume import Volume
 
 logger = logging.getLogger(__name__)
 
@@ -45,43 +45,6 @@ class Basis:
 
         self._build()
 
-    def _getfbzeros(self):
-        """
-        Generate zeros of Bessel functions
-        """
-        # get upper_bound of zeros of Bessel functions
-        upper_bound = min(self.ell_max + 1, 2 * self.nres + 1)
-
-        # List of number of zeros
-        n = []
-        # List of zero values (each entry is an ndarray; all of possibly different lengths)
-        zeros = []
-
-        # generate zeros of Bessel functions for each ell
-        for ell in range(upper_bound):
-            _n, _zeros = num_besselj_zeros(
-                ell + (self.ndim - 2) / 2, self.nres * np.pi / 2
-            )
-            if _n == 0:
-                break
-            else:
-                n.append(_n)
-                zeros.append(_zeros)
-
-        #  get maximum number of ell
-        self.ell_max = len(n) - 1
-
-        #  set the maximum of k for each ell
-        self.k_max = np.array(n, dtype=int)
-
-        max_num_zeros = max(len(z) for z in zeros)
-        for i, z in enumerate(zeros):
-            zeros[i] = np.hstack(
-                (z, np.zeros(max_num_zeros - len(z), dtype=self.dtype))
-            )
-
-        self.r0 = m_reshape(np.hstack(zeros), (-1, self.ell_max + 1)).astype(self.dtype)
-
     def _build(self):
         """
         Build the internal data structure to represent basis
@@ -174,16 +137,20 @@ class Basis:
             those first dimensions of `x`.
 
         """
-        # ensure the first dimensions with size of self.sz
+
+        if isinstance(x, Image) or isinstance(x, Volume):
+            x = x.asnumpy()
+
+        # check that last ndim values of input shape match
+        # the shape of this basis
+        assert (
+            x.shape[-self.ndim :] == self.sz
+        ), f"Last {self.ndim} dimensions of x must match {self.sz}."
+        # extract number of images/volumes, or () if only one
         sz_roll = x.shape[: -self.ndim]
-
+        # convert to standardized shape e.g. (L,L) to (1,L,L)
         x = x.reshape((-1, *self.sz))
 
-        ensure(
-            x.shape[-self.ndim :] == self.sz,
-            f"Last {self.ndim} dimensions of x must match {self.sz}.",
-        )
-
         operator = LinearOperator(
             shape=(self.count, self.count),
             matvec=lambda v: self.evaluate_t(self.evaluate(v)),
@@ -206,5 +173,5 @@ class Basis:
                 raise RuntimeError("Unable to converge!")
 
         # return v coefficients with the last dimension of self.count
-        v = v.reshape((-1, *sz_roll))
+        v = v.reshape((*sz_roll, -1))
         return v
