@@ -6,11 +6,9 @@ from sklearn.neighbors import NearestNeighbors
 from tqdm import tqdm
 
 from aspire.basis import FSPCABasis
-from aspire.classification import BFRAlign2D, Class2D
+from aspire.classification import BFSReddyChatterjiAverager2D, Class2D
 from aspire.classification.legacy_implementations import bispec_2drot_large, pca_y
-from aspire.image import Image
 from aspire.numeric import ComplexPCA
-from aspire.source import ArrayImageSource
 from aspire.utils.random import rand
 
 logger = logging.getLogger(__name__)
@@ -31,7 +29,7 @@ class RIRClass2D(Class2D):
         large_pca_implementation="legacy",
         nn_implementation="legacy",
         bispectrum_implementation="legacy",
-        aligner=None,
+        averager=None,
         dtype=None,
         seed=None,
     ):
@@ -48,7 +46,8 @@ class RIRClass2D(Class2D):
         Z. Zhao, Y. Shkolnisky, A. Singer, Rotationally Invariant Image Representation
         for Viewing Direction Classification in Cryo-EM. (2014)
 
-        :param src: Source instance
+        :param src: Source instance.  Note it is possible to use one `source` for classification (ie CWF),
+        and a different `source` for stacking in the `averager`.
         :param pca_basis: Optional FSPCA Basis instance
         :param fspca_components: Components (top eigvals) to keep from full FSCPA, default truncates to  400.
         :param alpha: Amplitude Power Scale, default 1/3 (eq 20 from  RIIR paper).
@@ -60,7 +59,7 @@ class RIRClass2D(Class2D):
         :param large_pca_implementation: See `pca`.
         :param nn_implementation: See `nn_classification`.
         :param bispectrum_implementation: See `bispectrum`.
-        :param aligner: An Align2D subclass. Defaults to BFRAlign2D.
+        :param averager: An Averager2D subclass. Defaults to BFSReddyChatterjiAverager2D.
         :param dtype: Optional dtype, otherwise taken from src.
         :param seed: Optional RNG seed to be passed to random methods, (example Random NN).
         :return: RIRClass2D instance to be used to compute bispectrum-like rotationally invariant 2D classification.
@@ -101,7 +100,7 @@ class RIRClass2D(Class2D):
         self.alpha = alpha
         self.bispectrum_components = bispectrum_components
         self.bispectrum_freq_cutoff = bispectrum_freq_cutoff
-        self.aligner = aligner
+        self.averager = averager
 
         if self.src.n < self.bispectrum_components:
             raise RuntimeError(
@@ -165,16 +164,17 @@ class RIRClass2D(Class2D):
         #  default of 400 components was taken from legacy code.
         # Instantiate a new compressed (truncated) basis.
         if self.pca_basis is None:
-            # self.pca_basis = self.pca_basis.compress(self.fspca_components)
             self.pca_basis = FSPCABasis(self.src, components=self.fspca_components)
 
         # For convenience, assign the fb_basis used in the pca_basis.
         self.fb_basis = self.pca_basis.basis
 
-        # When not provided by a user, the aligner is instantiated after
+        # When not provided by a user, the averager is instantiated after
         #  we are certain our pca_basis has been constructed.
-        if self.aligner is None:
-            self.aligner = BFRAlign2D(self.pca_basis, dtype=self.dtype)
+        if self.averager is None:
+            self.averager = BFSReddyChatterjiAverager2D(
+                self.fb_basis, self.src, dtype=self.dtype
+            )
 
         # Get the expanded coefs in the compressed FSPCA space.
         self.fspca_coef = self.pca_basis.spca_coef
@@ -184,7 +184,7 @@ class RIRClass2D(Class2D):
 
         # # Stage 2: Compute Nearest Neighbors
         logger.info("Calculate Nearest Neighbors")
-        classes, refl, distances = self.nn_classification(coef_b, coef_b_r)
+        classes, reflections, distances = self.nn_classification(coef_b, coef_b_r)
 
         if diagnostics:
             # Lets peek at the distribution of distances
@@ -193,24 +193,39 @@ class RIRClass2D(Class2D):
             plt.show()
 
             # Report some information about reflections
-            logger.info(f"Count reflected: {np.sum(refl)}" f" {100 * np.mean(refl) } %")
+            logger.info(
+                f"Count reflected: {np.sum(reflections)}"
+                f" {100 * np.mean(reflections) } %"
+            )
 
+        return classes, reflections, distances
+
+    def averages(self, classes, reflections, distances):
         # # Stage 3: Class Selection
-        logger.info(f"Select {self.n_classes} Classes from Nearest Neighbors")
         # This is an area open to active research.
         # Currently we take a naive approach by selecting the
         # first n_classes assuming they are quasi random.
-        classes = classes[: self.n_classes]
+        logger.info(f"Select {self.n_classes} Classes from Nearest Neighbors")
+        classes, reflections = self.select_classes(classes, reflections)
 
-        # # Stage 4: Align
+        # # Stage 4: Averager
         logger.info(
-            f"Begin Rotational Alignment of {classes.shape[0]} Classes using {self.aligner}."
+            f"Begin Averaging of {classes.shape[0]} Classes using {self.averager}."
         )
-        if not self.aligner.basis == self.pca_basis:
-            raise RuntimeError(
-                f"Aligner {self.aligner} basis does not match FSPCA basis."
-            )
-        return self.aligner.align(classes, refl, self.fspca_coef)
+
+        return self.averager.average(classes, reflections)
+
+    def select_classes(self, classes, reflections):
+        """
+        Select the `n_classes` to average from the (n_images) population of classes.
+        """
+        # Generate indices for random sample (can do something smarter, or build this out later).
+        # For testing/poc just take the first n_classes so it matches earlier plots for manual comparison
+        # If image stack is assumed to be reasonably random, this is a reasonable thing to do.
+        # Another reasonable thing would be to take a random selection over the whole dataset,
+        # in case the head of a dataset is too similar or has artifacts.
+        selection = np.arange(self.n_classes)
+        return classes[selection], reflections[selection]
 
     def pca(self, M):
         """
@@ -298,64 +313,9 @@ class RIRClass2D(Class2D):
 
         return indices
 
-    def output(
-        self,
-        classes,
-        classes_refl,
-        rot,
-        shifts=None,
-        coefs=None,
-    ):
-        """
-        Return class averages.
-
-        :param classes: class indices (refering to src). (n_img, n_nbor)
-        :param classes_refl: Bool representing whether to reflect image in `classes`
-        :param rot: Array of in-plane rotation angles (Radians) of image in `classes`
-        :param shifts: Optional array of shifts for image in `classes`.
-        :coefs: Optional Fourier bessel coefs (avoids recomputing).
-        :return: Stack of Synthetic Class Average images as Image instance.
-        """
-
-        logger.info(f"Select {self.n_classes} Classes from Nearest Neighbors")
-        # generate indices for random sample (can do something smart with corr later).
-        # For testing just take the first n_classes so it matches earlier plots for manual comparison
-        # This is assumed to be reasonably random.
-        selection = np.arange(self.n_classes)
-
-        imgs = self.src.images(0, self.src.n)
-        fb_avgs = np.empty((self.n_classes, self.fb_basis.count), dtype=self.src.dtype)
-
-        for i in tqdm(range(self.n_classes)):
-            j = selection[i]
-            # Get the neighbors
-            neighbors_ids = classes[j]
-
-            # Get coefs in Fourier Bessel Basis if not provided as an argument.
-            if coefs is None:
-                neighbors_imgs = Image(imgs[neighbors_ids])
-                if shifts is not None:
-                    neighbors_imgs.shift(shifts[i])
-                neighbors_coefs = self.fb_basis.evaluate_t(neighbors_imgs)
-            else:
-                neighbors_coefs = coefs[neighbors_ids]
-                if shifts is not None:
-                    neighbors_coefs = self.fb_basis.shift(neighbors_coefs, shifts[i])
-
-            # Rotate in Fourier Bessel
-            neighbors_coefs = self.fb_basis.rotate(
-                neighbors_coefs, rot[j], classes_refl[j]
-            )
-
-            # Averaging in FB
-            fb_avgs[i] = np.mean(neighbors_coefs, axis=0)
-
-        # Now we convert the averaged images from FB to Cartesian.
-        return ArrayImageSource(self.fb_basis.evaluate(fb_avgs))
-
     def _legacy_nn_classification(self, coeff_b, coeff_b_r, batch_size=2000):
         """
-        Perform nearest neighbor classification and alignment.
+        Perform nearest neighbor classification.
         """
 
         # Note kept ordering from legacy code (n_features, n_img)
@@ -390,7 +350,7 @@ class RIRClass2D(Class2D):
             # Check with Joakim about preference.
             # I (GBW) think class[i] should have class[i][0] be the original image index.
             classes[start:finish] = np.argsort(-corr, axis=1)[:, :n_nbor]
-            # Store the corr values for the n_nhors in this batch
+            # Store the corr values for the n_nbors in this batch
             distances[start:finish] = np.take_along_axis(
                 corr, classes[start:finish], axis=1
             )
