@@ -29,7 +29,41 @@ logger = logging.getLogger(__name__)
     type=int,
     help="Max number of image rows to read from STAR file",
 )
-def orient3d(data_folder, starfile_in, starfile_out, pixel_size, max_rows):
+@click.option(
+    "--n_rad",
+    default=None,
+    type=int,
+    help="Number of points in the radial direction. If None, defaults to half the resolution of the source.",
+)
+@click.option(
+    "--n_theta",
+    default=360,
+    type=int,
+    help="Number of points in the theta direction",
+)
+@click.option(
+    "--max_shift",
+    default=0.15,
+    type=float,
+    help="Maximum range for shifts as a proportion of resolution",
+)
+@click.option(
+    "--shift_step",
+    default=1,
+    type=int,
+    help="Resolution for shift estimation in pixels",
+)
+def orient3d(
+    data_folder,
+    starfile_in,
+    starfile_out,
+    pixel_size,
+    max_rows,
+    n_rad,
+    n_theta,
+    max_shift,
+    shift_step,
+):
     """
     Input images from STAR file and estimate orientational angles
     """
@@ -41,7 +75,9 @@ def orient3d(data_folder, starfile_in, starfile_out, pixel_size, max_rows):
 
     # Estimate rotation matrices
     logger.info("Estimate rotation matrices.")
-    orient_est = CLSyncVoting(source)
+    orient_est = CLSyncVoting(
+        source, n_rad=n_rad, n_theta=n_theta, max_shift=max_shift, shift_step=shift_step
+    )
     orient_est.estimate_rotations()
 
     # Create new source object and save Estimate rotation matrices
