@@ -6,7 +6,7 @@ from scipy.linalg import solve, sqrtm
 
 from aspire.operators import BlkDiagMatrix, RadialCTFFilter
 from aspire.optimization import conj_grad, fill_struct
-from aspire.utils import ensure, make_symmat
+from aspire.utils import make_symmat
 from aspire.utils.matlab_compat import m_reshape
 
 logger = logging.getLogger(__name__)
@@ -22,10 +22,11 @@ def shrink_covar(covar, noise_var, gamma, shrinker="frobenius_norm"):
     :return: The shrinked covariance matrix
     """
 
-    ensure(
-        shrinker in ("frobenius_norm", "operator_norm", "soft_threshold"),
-        "Unsupported shrink method",
-    )
+    assert shrinker in (
+        "frobenius_norm",
+        "operator_norm",
+        "soft_threshold",
+    ), "Unsupported shrink method"
 
     lambs, eig_vec = eig(make_symmat(covar))
 
@@ -64,7 +65,7 @@ def shrink_covar(covar, noise_var, gamma, shrinker="frobenius_norm"):
             - noise_var
         )
         c = np.divide(
-            (1 - np.divide(noise_var ** 2 * gamma, lambdas ** 2)),
+            (1 - np.divide(noise_var**2 * gamma, lambdas**2)),
             (1 + np.divide(noise_var * gamma, lambdas)),
         )
         lambdas = lambdas * c
@@ -96,7 +97,7 @@ class RotCov2D:
         """
         self.basis = basis
         self.dtype = self.basis.dtype
-        ensure(basis.ndim == 2, "Only two-dimensional basis functions are needed.")
+        assert basis.ndim == 2, "Only two-dimensional basis functions are needed."
 
     def _get_mean(self, coeffs):
         """
@@ -284,7 +285,7 @@ class RotCov2D:
 
         for k in np.unique(ctf_idx[:]):
 
-            coeff_k = coeffs[ctf_idx == k]
+            coeff_k = coeffs[ctf_idx == k].astype(self.dtype)
             weight = coeff_k.shape[0] / coeffs.shape[0]
 
             ctf_fb_k = ctf_fb[k]
@@ -327,10 +328,10 @@ class RotCov2D:
 
         def precond_fun(S, x):
             p = np.size(S, 0)
-            ensure(np.size(x) == p * p, "The sizes of S and x are not consistent.")
+            assert np.size(x) == p * p, "The sizes of S and x are not consistent."
             x = m_reshape(x, (p, p))
             y = S @ x @ S
-            y = m_reshape(y, (p ** 2,))
+            y = m_reshape(y, (p**2,))
             return y
 
         def apply(A, x):
@@ -339,7 +340,7 @@ class RotCov2D:
             y = np.zeros_like(x)
             for k in range(0, len(A)):
                 y = y + A[k] @ x @ A[k].T
-            y = m_reshape(y, (p ** 2,))
+            y = m_reshape(y, (p**2,))
             return y
 
         for ell in range(0, len(b)):
@@ -347,7 +348,7 @@ class RotCov2D:
             for k in range(0, len(A)):
                 A_ell.append(A[k][ell])
             p = np.size(A_ell[0], 0)
-            b_ell = m_reshape(b[ell], (p ** 2,))
+            b_ell = m_reshape(b[ell], (p**2,))
             S = inv(M[ell])
             cg_opt["preconditioner"] = lambda x: precond_fun(S, x)
             covar_coeff_ell, _, _ = conj_grad(lambda x: apply(A_ell, x), b_ell, cg_opt)
@@ -632,10 +633,10 @@ class BatchedRotCov2D(RotCov2D):
 
         def precond_fun(S, x):
             p = np.size(S, 0)
-            ensure(np.size(x) == p * p, "The sizes of S and x are not consistent.")
+            assert np.size(x) == p * p, "The sizes of S and x are not consistent."
             x = m_reshape(x, (p, p))
             y = S @ x @ S
-            y = m_reshape(y, (p ** 2,))
+            y = m_reshape(y, (p**2,))
             return y
 
         def apply(A, x):
@@ -644,7 +645,7 @@ class BatchedRotCov2D(RotCov2D):
             y = np.zeros_like(x)
             for k in range(0, len(A)):
                 y = y + A[k] @ x @ A[k].T
-            y = m_reshape(y, (p ** 2,))
+            y = m_reshape(y, (p**2,))
             return y
 
         cg_opt = covar_est_opt
@@ -655,7 +656,7 @@ class BatchedRotCov2D(RotCov2D):
             for k in range(0, len(A_covar)):
                 A_ell.append(A_covar[k][ell])
             p = np.size(A_ell[0], 0)
-            b_ell = m_reshape(b_covar[ell], (p ** 2,))
+            b_ell = m_reshape(b_covar[ell], (p**2,))
             S = inv(M[ell])
             cg_opt["preconditioner"] = lambda x: precond_fun(S, x)
             covar_coeff_ell, _, _ = conj_grad(lambda x: apply(A_ell, x), b_ell, cg_opt)
