@@ -21,8 +21,7 @@ from aspire.image import Image
 from aspire.numeric import fft
 from aspire.operators import voltage_to_wavelength
 from aspire.storage import StarFile
-from aspire.utils import abs2, complex_type
-from aspire.utils.coor_trans import grid_1d, grid_2d
+from aspire.utils import abs2, complex_type, grid_1d, grid_2d
 
 logger = logging.getLogger(__name__)
 
@@ -63,14 +62,12 @@ class CtfEstimator:
         self.lmbd = voltage_to_wavelength(voltage) / 10.0  # (Angstrom)
         self.dtype = np.dtype(dtype)
 
-        grid = grid_2d(psd_size, normalized=True, dtype=self.dtype)
+        grid = grid_2d(psd_size, normalized=True, indexing="yx", dtype=self.dtype)
 
-        # Note this mesh for x,y is transposed, and range is -half to half.
-        rb = np.sqrt((grid["x"] / 2) ** 2 + (grid["y"] / 2) ** 2).T
+        # Note range is -half to half.
+        self.r_ctf = grid["r"] / 2 * (10 / pixel_size)  # units: inverse nm
 
-        self.r_ctf = rb * (10 / pixel_size)  # units: inverse nm
-        # Note this mesh for theta is transposed.
-        self.theta = grid["phi"].T
+        self.theta = grid["phi"]
         self.defocus1 = 0
         self.defocus2 = 0
         self.angle = 0  # Radians
@@ -124,12 +121,12 @@ class CtfEstimator:
         )
         defocus_factor = np.pi * self.lmbd * self.r_ctf * defocus / 2
         amplitude_contrast_term = self.amplitude_contrast / np.sqrt(
-            1 - self.amplitude_contrast ** 2
+            1 - self.amplitude_contrast**2
         )
 
         chi = (
             defocus_factor
-            - np.pi * self.lmbd ** 3 * self.cs * 1e6 * self.r_ctf ** 2 / 2
+            - np.pi * self.lmbd**3 * self.cs * 1e6 * self.r_ctf**2 / 2
             + amplitude_contrast_term
         )
         h = -np.sin(chi)
@@ -180,7 +177,7 @@ class CtfEstimator:
         # Create a sum and reshape so it may be broadcast with `block`.
         blocks_sum = np.sum(blocks, axis=(-1, -2))[:, np.newaxis, np.newaxis]
 
-        blocks -= blocks_sum / (block_size ** 2)
+        blocks -= blocks_sum / (block_size**2)
 
         return blocks
 
@@ -406,7 +403,7 @@ class CtfEstimator:
         center = N // 2
 
         grid = grid_1d(N, normalized=True, dtype=self.dtype)
-        rb = grid["x"][0][center:] / 2
+        rb = grid["x"][center:] / 2
 
         r_ctf = rb * (10 / pixel_size)  # units: inverse nm
 
@@ -415,14 +412,14 @@ class CtfEstimator:
         signal = np.sqrt(signal)
         signal = signal[: 3 * signal.shape[0] // 4]
 
-        r_ctf_sq = r_ctf ** 2
+        r_ctf_sq = r_ctf**2
         c = np.zeros((max_defocus - min_defocus, signal.shape[1]), dtype=self.dtype)
 
         for f in range(min_defocus, max_defocus):
             ctf_im = np.abs(
                 np.sin(
                     np.pi * lmbd * f * r_ctf_sq
-                    - 0.5 * np.pi * (lmbd ** 3) * cs * 1e6 * r_ctf_sq ** 2
+                    - 0.5 * np.pi * (lmbd**3) * cs * 1e6 * r_ctf_sq**2
                     + w
                 )
             )
@@ -437,8 +434,8 @@ class CtfEstimator:
                 ctf_im[: m + 1, m] = np.zeros((m + 1))
                 signal[: m + 1, m] = np.zeros((m + 1))
 
-            Sx = np.sqrt(np.sum(ctf_im ** 2, axis=0))
-            Sy = np.sqrt(np.sum(signal ** 2, axis=0))
+            Sx = np.sqrt(np.sum(ctf_im**2, axis=0))
+            Sy = np.sqrt(np.sum(signal**2, axis=0))
             c[f - min_defocus, :] = np.sum(ctf_im * signal, axis=0) / (Sx * Sy)
 
         avg_defocus, low_freq_cutoff = np.unravel_index(np.argmax(c), c.shape)[:2]
@@ -459,9 +456,9 @@ class CtfEstimator:
         signal = signal.asnumpy()
 
         N = signal.shape[1]
-        grid = grid_2d(N, normalized=False, dtype=self.dtype)
+        grid = grid_2d(N, normalized=False, indexing="yx", dtype=self.dtype)
 
-        radii = np.sqrt((grid["x"] / 2) ** 2 + (grid["y"] / 2) ** 2).T
+        radii = np.sqrt((grid["x"] / 2) ** 2 + (grid["y"] / 2) ** 2)
 
         background = np.zeros(signal.shape, dtype=self.dtype)
         for r in range(max_col + 2, background_p1.shape[1]):
@@ -490,14 +487,13 @@ class CtfEstimator:
         N = signal.shape[0]
         center = N // 2
 
-        grid = grid_2d(N, normalized=True, dtype=self.dtype)
-        rb = np.sqrt((grid["x"] / 2) ** 2 + (grid["y"] / 2) ** 2).T
+        grid = grid_2d(N, normalized=True, indexing="yx", dtype=self.dtype)
 
-        r_ctf = rb * (10 / pixel_size)
+        r_ctf = grid["r"] / 2 * (10 / pixel_size)
 
-        grid = grid_2d(N, normalized=False, dtype=self.dtype)
-        X = grid["x"].T
-        Y = grid["y"].T
+        grid = grid_2d(N, normalized=False, indexing="yx", dtype=self.dtype)
+        X = grid["x"]
+        Y = grid["y"]
 
         signal -= np.min(signal)
 
@@ -510,13 +506,13 @@ class CtfEstimator:
         max_limit = r_ctf[center, (center + np.ceil(rad_sq_max)).astype(int)]
         signal = np.where(r_ctf > max_limit, 0, signal)
 
-        moment_02 = Y ** 2 * signal
+        moment_02 = Y**2 * signal
         moment_02 = np.sum(moment_02, axis=(0, 1))
 
         moment_11 = Y * X * signal
         moment_11 = np.sum(moment_11, axis=(0, 1))
 
-        moment_20 = X ** 2 * signal
+        moment_20 = X**2 * signal
         moment_20 = np.sum(moment_20, axis=(0, 1))
 
         moment_mat = np.zeros((2, 2))
@@ -572,8 +568,8 @@ class CtfEstimator:
         y = (df1 - df2) * np.cos(2 * angle_ast)
         z = (df1 - df2) * np.sin(2 * angle_ast)
 
-        a = np.pi * lmbd * r ** 2 / 2
-        b = np.pi * lmbd ** 3 * cs * 1e6 * r ** 4 / 2 - np.full(
+        a = np.pi * lmbd * r**2 / 2
+        b = np.pi * lmbd**3 * cs * 1e6 * r**4 / 2 - np.full(
             shape=r.shape, fill_value=amplitude_contrast, dtype=self.dtype
         )
 
@@ -594,7 +590,7 @@ class CtfEstimator:
         r = r[mask]
         theta = theta[mask]
 
-        sum_A = np.sum(signal ** 2)
+        sum_A = np.sum(signal**2)
 
         dx = 1
         dy = 1
@@ -614,7 +610,7 @@ class CtfEstimator:
             sine_z_term = a * np.sin(2 * theta)
 
             c1 = np.sum(np.abs(outer_sine) * signal)
-            c2 = np.sqrt(sum_A * np.sum(outer_sine ** 2))
+            c2 = np.sqrt(sum_A * np.sum(outer_sine**2))
 
             # gradients of numerator
             dx_c1 = np.sum(np.sign(outer_sine) * outer_cosine * a * signal)
@@ -626,7 +622,7 @@ class CtfEstimator:
                 np.sign(outer_sine) * outer_cosine * a * np.sin(2 * theta) * signal
             )
 
-            derivative_sqrt = 1 / (2 * np.sqrt(sum_A * np.sum(outer_sine ** 2)))
+            derivative_sqrt = 1 / (2 * np.sqrt(sum_A * np.sum(outer_sine**2)))
 
             derivative_sine2 = 2 * outer_sine * outer_cosine
 
@@ -636,9 +632,9 @@ class CtfEstimator:
             dz_c2 = derivative_sqrt * sum_A * np.sum(derivative_sine2 * sine_z_term)
 
             # gradients
-            dx = (dx_c1 * c2 - dx_c2 * c1) / c2 ** 2
-            dy = (dy_c1 * c2 - dy_c2 * c1) / c2 ** 2
-            dz = (dz_c1 * c2 - dz_c2 * c1) / c2 ** 2
+            dx = (dx_c1 * c2 - dx_c2 * c1) / c2**2
+            dy = (dy_c1 * c2 - dy_c2 * c1) / c2**2
+            dz = (dz_c1 * c2 - dz_c2 * c1) / c2**2
 
             # update
             x = x + alpha1 * dx
@@ -666,7 +662,7 @@ class CtfEstimator:
         sine_z_term = a * np.sin(2 * theta)
 
         c1 = np.sum(np.abs(outer_sine) * signal)
-        c2 = np.sqrt(sum_A * np.sum(outer_sine ** 2))
+        c2 = np.sqrt(sum_A * np.sum(outer_sine**2))
 
         p = c1 / c2
 
@@ -727,7 +723,7 @@ def estimate_ctf(
 
     amp = amplitude_contrast
     amplitude_contrast = np.arctan(
-        amplitude_contrast / np.sqrt(1 - amplitude_contrast ** 2)
+        amplitude_contrast / np.sqrt(1 - amplitude_contrast**2)
     )
 
     lmbd = voltage_to_wavelength(voltage) / 10  # (Angstrom)
@@ -791,11 +787,10 @@ def estimate_ctf(
         initial_df1 = (avg_defocus * 2) / (1 + ratio)
         initial_df2 = (avg_defocus * 2) - initial_df1
 
-        grid = grid_2d(psd_size, normalized=True, dtype=dtype)
+        grid = grid_2d(psd_size, normalized=True, indexing="yx", dtype=dtype)
 
-        rb = np.sqrt((grid["x"] / 2) ** 2 + (grid["y"] / 2) ** 2).T
-        r_ctf = rb * (10 / pixel_size)
-        theta = grid["phi"].T
+        r_ctf = grid["r"] / 2 * (10 / pixel_size)
+        theta = grid["phi"]
 
         angle = -5 / 12 * np.pi  # Radians (-75 degrees)
         cc_array = np.zeros((6, 4))
@@ -851,7 +846,7 @@ def estimate_ctf(
             cc_array[ml, 0] - cc_array[ml, 1]
         ) * np.cos(2 * theta - 2 * cc_array[ml, 2] * np.ones(theta.shape, theta.dtype))
         ctf_im = -np.sin(
-            np.pi * lmbd * r_ctf ** 2 / 2 * (df - lmbd ** 2 * r_ctf ** 2 * cs * 1e6)
+            np.pi * lmbd * r_ctf**2 / 2 * (df - lmbd**2 * r_ctf**2 * cs * 1e6)
             + amplitude_contrast
         )
         ctf_signal = np.zeros(ctf_im.shape, ctf_im.dtype)
