@@ -5,7 +5,6 @@ from collections import OrderedDict
 import mrcfile
 import numpy as np
 import pandas as pd
-from scipy.spatial.transform import Rotation as R
 
 from aspire.image import Image, normalize_bg
 from aspire.image.xform import (
@@ -23,8 +22,7 @@ from aspire.operators import (
     PowerFilter,
 )
 from aspire.storage import MrcStats, StarFile
-from aspire.utils import ensure
-from aspire.utils.coor_trans import grid_2d
+from aspire.utils import Rotation, grid_2d
 
 logger = logging.getLogger(__name__)
 
@@ -45,49 +43,10 @@ class ImageSource:
     `Filter` objects, for example, are stored in this metadata table as references to unique `Filter` objects that
     correspond to images in this `ImageSource`. Several rows of metadata may end up containing a reference to a small
     handful of unique `Filter` objects, depending on the values found in other columns (identical `Filter`
-    objects, depending on unique CTF values found for _rlnDefocusU/_rlnDefocusV etc.
+    objects). For example, a smaller number of CTFFilter objects may apply to subsets of particles depending on
+    the unique "_rlnDefocusU"/"_rlnDefocusV" Relion parameters.
     """
 
-    """
-    The metadata_fields dictionary below specifies default data types of certain key fields used in the codebase.
-    The STAR file used to initialize subclasses of ImageSource may well contain other columns not found below; these
-    additional columns are available when read, and they default to the pandas data type 'object'.
-    """
-    metadata_fields = {
-        "_rlnVoltage": float,
-        "_rlnDefocusU": float,
-        "_rlnDefocusV": float,
-        "_rlnDefocusAngle": float,
-        "_rlnSphericalAberration": float,
-        "_rlnDetectorPixelSize": float,
-        "_rlnCtfFigureOfMerit": float,
-        "_rlnMagnification": float,
-        "_rlnAmplitudeContrast": float,
-        "_rlnImageName": str,
-        "_rlnOriginalName": str,
-        "_rlnCtfImage": str,
-        "_rlnCoordinateX": float,
-        "_rlnCoordinateY": float,
-        "_rlnCoordinateZ": float,
-        "_rlnNormCorrection": float,
-        "_rlnMicrographName": str,
-        "_rlnGroupName": str,
-        "_rlnGroupNumber": str,
-        "_rlnOriginX": float,
-        "_rlnOriginY": float,
-        "_rlnAngleRot": float,
-        "_rlnAngleTilt": float,
-        "_rlnAnglePsi": float,
-        "_rlnClassNumber": int,
-        "_rlnLogLikeliContribution": float,
-        "_rlnRandomSubset": int,
-        "_rlnParticleName": str,
-        "_rlnOriginalParticleName": str,
-        "_rlnNrOfSignificantSamples": float,
-        "_rlnNrOfFrames": int,
-        "_rlnMaxValueProbDistribution": float,
-    }
-
     def __init__(self, L, n, dtype="double", metadata=None, memory=None):
         """
         A Cryo-EM ImageSource object that supplies images along with other parameters for image manipulation.
@@ -117,18 +76,26 @@ class ImageSource:
         else:
             self._metadata = metadata
             if self.has_metadata(["_rlnAngleRot", "_rlnAngleTilt", "_rlnAnglePsi"]):
-                self._rotations = R.from_euler(
-                    "ZYZ",
-                    self.get_metadata(
-                        ["_rlnAngleRot", "_rlnAngleTilt", "_rlnAnglePsi"]
-                    ),
-                    degrees=True,
+                self._rotations = Rotation.from_euler(
+                    np.deg2rad(
+                        self.get_metadata(
+                            ["_rlnAngleRot", "_rlnAngleTilt", "_rlnAnglePsi"]
+                        )
+                    )
                 )
 
         self.unique_filters = []
         self.generation_pipeline = Pipeline(xforms=None, memory=memory)
         self._metadata_out = None
 
+        logger.info(f"Creating {self.__class__.__name__} with {len(self)} images.")
+
+    def __len__(self):
+        """
+        Returns total number of images in source.
+        """
+        return self.n
+
     @property
     def states(self):
         return np.atleast_1d(self.get_metadata("_rlnClassNumber"))
@@ -139,39 +106,11 @@ class ImageSource:
 
     @property
     def filter_indices(self):
-        return self.get_metadata("__filter_indices")
+        return np.atleast_1d(self.get_metadata("__filter_indices"))
 
     @filter_indices.setter
     def filter_indices(self, indices):
         # create metadata of filters for all images
-        if indices is None:
-            filter_values = np.nan
-        else:
-            attribute_list = (
-                "voltage",
-                "defocus_u",
-                "defocus_v",
-                "defocus_ang",
-                "Cs",
-                "alpha",
-            )
-            filter_values = np.zeros((len(indices), len(attribute_list)))
-            for i, filt in enumerate(self.unique_filters):
-                filter_values[indices == i] = [
-                    getattr(filt, attribute, np.nan) for attribute in attribute_list
-                ]
-
-        self.set_metadata(
-            [
-                "_rlnVoltage",
-                "_rlnDefocusU",
-                "_rlnDefocusV",
-                "_rlnDefocusAngle",
-                "_rlnSphericalAberration",
-                "_rlnAmplitudeContrast",
-            ],
-            filter_values,
-        )
         return self.set_metadata(["__filter_indices"], indices)
 
     @property
@@ -204,7 +143,7 @@ class ImageSource:
         """
         Converts internal _rotations representation to expected matrix form.
         """
-        return self._rotations.as_euler("ZYZ", degrees=False).astype(self.dtype)
+        return self._rotations.angles.astype(self.dtype)
 
     @property
     def rots(self):
@@ -219,7 +158,7 @@ class ImageSource:
         Converts internal `_rotations` representation to expected matrix form.
         :return: Rotation matrices as a n x 3 x 3 array
         """
-        return self._rotations.as_matrix().astype(self.dtype)
+        return self._rotations.matrices.astype(self.dtype)
 
     @angles.setter
     def angles(self, values):
@@ -228,7 +167,7 @@ class ImageSource:
         :param values: Rotation angles in radians, as a n x 3 array
         :return: None
         """
-        self._rotations = R.from_euler("ZYZ", values)
+        self._rotations = Rotation.from_euler(values)
         self.set_metadata(
             ["_rlnAngleRot", "_rlnAngleTilt", "_rlnAnglePsi"], np.rad2deg(values)
         )
@@ -240,10 +179,10 @@ class ImageSource:
         :param values: Rotation matrices as a n x 3 x 3 array
         :return: None
         """
-        self._rotations = R.from_matrix(values)
+        self._rotations = Rotation.from_matrix(values)
         self.set_metadata(
             ["_rlnAngleRot", "_rlnAngleTilt", "_rlnAnglePsi"],
-            self._rotations.as_euler("ZYZ", degrees=True),
+            np.rad2deg(self._rotations.angles),
         )
 
     def set_metadata(self, metadata_fields, values, indices=None):
@@ -339,42 +278,41 @@ class ImageSource:
             "Subclasses should implement this and return an Image object"
         )
 
-    def eval_filters(self, im_orig, start=0, num=np.inf, indices=None):
+    def _apply_filters(
+        self,
+        im_orig,
+        filters,
+        indices,
+    ):
+        """
+        For images in `im_orig`, `filters` associated with the corresponding
+        index in the supplied `indices` are applied. The images are then returned as an `Image` stack.
+        :param im_orig: An `Image` object
+        :param filters: A list of `Filter` objects
+        :param indices: A list of indices indicating the corresponding filter in `filters`
+        """
         if not isinstance(im_orig, Image):
             logger.warning(
-                f"eval_filters passed {type(im_orig)} instead of Image instance"
+                f"_apply_filters() passed {type(im_orig)} instead of Image instance"
             )
             # for now just convert it
             im = Image(im_orig)
 
         im = im_orig.copy()
 
-        if indices is None:
-            indices = np.arange(start, min(start + num, self.n))
-
-        for i, filt in enumerate(self.unique_filters):
-            idx_k = np.where(self.filter_indices[indices] == i)[0]
+        for i, filt in enumerate(filters):
+            idx_k = np.where(indices == i)[0]
             if len(idx_k) > 0:
                 im[idx_k] = Image(im[idx_k]).filter(filt).asnumpy()
 
         return im
 
-    def eval_filter_grid(self, L, power=1):
-        grid2d = grid_2d(L, dtype=self.dtype)
-        omega = np.pi * np.vstack((grid2d["x"].flatten(), grid2d["y"].flatten()))
-
-        h = np.empty((omega.shape[-1], len(self.filter_indices)), dtype=self.dtype)
-        for i, filt in enumerate(self.unique_filters):
-            idx_k = np.where(self.filter_indices == i)[0]
-            if len(idx_k) > 0:
-                filter_values = filt.evaluate(omega)
-                if power != 1:
-                    filter_values **= power
-                h[:, idx_k] = np.column_stack((filter_values,) * len(idx_k))
-
-        h = np.reshape(h, grid2d["x"].shape + (len(self.filter_indices),))
-
-        return h
+    def _apply_source_filters(self, im_orig, indices):
+        return self._apply_filters(
+            im_orig,
+            self.unique_filters,
+            self.filter_indices[indices],
+        )
 
     def cache(self):
         logger.info("Caching source images")
@@ -403,10 +341,9 @@ class ImageSource:
         return im
 
     def downsample(self, L):
-        ensure(
-            L <= self.L,
-            "Max desired resolution should be less than the current resolution",
-        )
+        assert (
+            L <= self.L
+        ), "Max desired resolution should be less than the current resolution"
         logger.info(f"Setting max. resolution of source = {L}")
 
         self.generation_pipeline.add_xform(Downsample(resolution=L))
@@ -465,7 +402,7 @@ class ImageSource:
 
         logger.info("Apply contrast inversion on source object")
         L = self.L
-        grid = grid_2d(L, shifted=True)
+        grid = grid_2d(L, indexing="yx", shifted=True)
         # Get mask indices of signal and noise samples assuming molecule
         signal_mask = grid["r"] < 0.5
         noise_mask = grid["r"] > 0.8
@@ -531,7 +468,7 @@ class ImageSource:
         all_idx = np.arange(start, min(start + num, self.n))
         im *= self.amplitudes[all_idx, np.newaxis, np.newaxis]
         im = im.shift(-self.offsets[all_idx, :])
-        im = self.eval_filters(im, start=start, num=num)
+        im = self._apply_source_filters(im, all_idx)
 
         vol = im.backproject(self.rots[start : start + num, :, :])[0]
 
@@ -553,7 +490,7 @@ class ImageSource:
             logger.warning(f"Volume.dtype {vol.dtype} inconsistent with {self.dtype}")
 
         im = vol.project(0, self.rots[all_idx, :, :])
-        im = self.eval_filters(im, start, num)
+        im = self._apply_source_filters(im, all_idx)
         im = im.shift(self.offsets[all_idx, :])
         im *= self.amplitudes[all_idx, np.newaxis, np.newaxis]
         return im
@@ -788,7 +725,7 @@ class ArrayImageSource(ImageSource):
 
         # Create filter indices, these are required to pass unharmed through filter eval code
         #   that is potentially called by other methods later.
-        self.filter_indices = np.zeros(self.n)
+        self.filter_indices = np.zeros(self.n, dtype=int)
         self.unique_filters = [IdentityFilter()]
 
         # Optionally populate angles/rotations.
