@@ -1,3 +1,3 @@
 from .micrograph import Micrograph
 from .mrc import MrcStats
-from .starfile import StarFile, StarFileBlock
+from .starfile import StarFile, StarFileError
